'use strict';

(function () {
  var hasProp = ({}).hasOwnProperty,
      getDesc = Object.getOwnPropertyDescriptor,
      defProp = Object.defineProperty;

  module.exports = function replace(cons, wrapper) {
    var name = arguments.length <= 2 || arguments[2] === undefined ? cons.name : arguments[2];
    return (function () {
      return copyProps(cons, named(name, cons, wrapper));
    })();
  };

  function copyProps(src, dst) {
    var keys = [];

    if (typeof Object.getOwnPropertyNames === 'function') {
      keys = keys.concat(Object.getOwnPropertyNames(src));
    }

    if (typeof Object.getOwnPropertySymbols === 'function') {
      keys = keys.concat(Object.getOwnPropertySymbols(src));
    }

    if (keys.length && getDesc != null && defProp != null) {
      for (var i = 0, len = keys.length; i < len; i++) {
        var k = keys[i],
            d = getDesc(dst, k);

        if ((d != null ? d.configurable : void 0) === false) {
          if (d.writable) dst[k] = src[k];
          continue;
        }
        try {
          defProp(dst, k, getDesc(src, k));
        } catch (_) {}
      }
    } else {
      for (var k in src) {
        if (!hasProp.call(src, k)) continue;
        dst[k] = src[k];
      }

      dst.prototype = src.prototype;
    }

    if (dst.__proto__ !== src.__proto__) {
      try {
        dst.__proto__ = src.__proto__;
      } catch (_) {}
    }

    dst.prototype.constructor = dst;
    return dst;
  }

  function named(name, src, dst) {
    if (!name) return dst;

    src = {
      name: name,
      length: src.length
    };

    for (var prop in ['name', 'length']) {
      if (dst[prop] !== src[prop]) {
        try {
          dst[prop] = src[prop];
        } catch (_) {}
      }

      if (dst[prop] !== src[prop]) {
        try {
          defProp(dst, prop, { value: src[prop] });
        } catch (_) {}
      }
    }

    if (dst.name !== name || dst.length !== src.length) {
      var args = !src.length ? '' : 'arg' + (function () {
        var results = [];

        for (var j = 1, l = src.length; 1 <= l ? j <= l : j >= l; 1 <= l ? j++ : j--) {
          results.push(j);
        }

        return results;
      })().join(', arg');

      try {
        dst = new Function('$$' + name, ("return function NAME(" + args + ") {\n    return $$NAME.apply(this, arguments);\n}").replace(/NAME/g, name))(dst);
      } catch (_) {}
    }

    return dst;
  }
}).call(undefined);
